using System;
using System.Linq;
using System.Net;
using System.Text.RegularExpressions;
using System.Web.Http;
using Orchard.Blogs.Models;
using Orchard.Blogs.Services;
using Orchard.BlogsApi.Models;
using Orchard.ContentManagement;
using Orchard.ContentManagement.Aspects;
using Orchard.DisplayManagement;
using Orchard.Localization;
using Orchard.Settings;
using Orchard.UI.Navigation;

namespace Orchard.BlogsApi.Controllers {

    [UseJsonp]
    public class BlogApiController : ApiController {
        private readonly IOrchardServices _services;
        private readonly IBlogService _blogService;
        private readonly IBlogPostService _blogPostService;
        private readonly ISiteService _siteService;

        public BlogApiController(
            IOrchardServices services,
            IBlogService blogService,
            IShapeFactory shapeFactory, IBlogPostService blogPostService, ISiteService siteService)
        {
            _services = services;
            _blogService = blogService;
            Shape = shapeFactory;
            _blogPostService = blogPostService;
            _siteService = siteService;
        }

        dynamic Shape { get; set; }

        [HttpGet]
        public Blog ListBlogPosts(int blogId, [FromUri]PagerParameters pagerParameters)
        {
            Pager pager = new Pager(_siteService.GetSiteSettings(), pagerParameters);

            var blogPart = _blogService.Get(blogId, VersionOptions.Published).As<BlogPart>();
            if (blogPart == null)
                throw new HttpResponseException(HttpStatusCode.NotFound);

            if (!_services.Authorizer.Authorize(Core.Contents.Permissions.ViewContent, blogPart, NullLocalizer.Instance("Cannot view content")))
            {
                throw new HttpResponseException(HttpStatusCode.Unauthorized);
            }

            var blog = _blogService.Get(blogId, VersionOptions.Published).As<BlogPart>();

            return new Blog
            {
                Title = blog.Name,
                Description = blog.Description,
                PostCount = blog.PostCount,
                Posts = _blogPostService.Get(blogPart, pager.GetStartIndex(), pager.PageSize).Select(bp => new BlogPost
                {
                    Text = ConvertRelativePathsToAbsolute(bp.Text, Request.RequestUri.Scheme + "://" + Request.RequestUri.Authority + "/"),
                    Title = bp.Title,
                    PublishedUtc = CreatedAt(bp)
                }).ToList()
            };
        }

        public static string ConvertRelativePathsToAbsolute(string text, string absoluteUrl)
        {
            string value = Regex.Replace(text, "<(.*?)(src|href)=\"(?!http)(.*?)\"(.*?)>", "<$1$2=\"" + absoluteUrl + "$3\"$4>",
                                         RegexOptions.IgnoreCase | RegexOptions.Multiline);

            // Now just make sure that there isn't a // because if
            // the original relative path started with a / then the
            // replacement above would create a //.

            return value.Replace(absoluteUrl + "/", absoluteUrl);
        }

        private static DateTime? CreatedAt(BlogPostPart bgp)
        {
            return bgp.As<ICommonPart>().CreatedUtc;
        }
    }
}
