﻿using System;
using System.IO;
using System.Net;
using System.Net.Http;
using System.Net.Http.Formatting;
using System.Net.Http.Headers;
using System.Threading.Tasks;
using System.Web;
using System.Web.Http.Controllers;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using Newtonsoft.Json.Serialization;


namespace Orchard.BlogsApi
{
    public class UseJsonpAttribute: Attribute, IControllerConfiguration
    {
        public void Initialize(HttpControllerSettings controllerSettings, HttpControllerDescriptor controllerDescriptor) {
            controllerSettings.Formatters.Insert(0, new JsonpFormatter());
        }
    }


    public class JsonpFormatter: JsonMediaTypeFormatter
    {
        public JsonpFormatter()
        {
            JsonpParameterName = "callback";
        }

        /// <summary>
        ///  Name of the query string parameter to look for
        ///  the jsonp function name
        /// </summary>
        public string JsonpParameterName { get; set; }

        /// <summary>
        /// Captured name of the Jsonp function that the JSON call
        /// is wrapped in. Set in GetPerRequestFormatter Instance
        /// </summary>
        private string _jsonpCallbackFunction;


        public override bool CanWriteType(Type type)
        {
            return true;
        }

        private static readonly CamelCasePropertyNamesContractResolver CamelCasePropertyNamesContractResolver = new CamelCasePropertyNamesContractResolver();


        /// <summary>
        /// Override this method to capture the Request object
        /// </summary>
        /// <param name="type"></param>
        /// <param name="request"></param>
        /// <param name="mediaType"></param>
        /// <returns></returns>
        public override MediaTypeFormatter GetPerRequestFormatterInstance(Type type, HttpRequestMessage request, MediaTypeHeaderValue mediaType)
        {
            var formatter = new JsonpFormatter
            {
                _jsonpCallbackFunction = GetJsonCallbackFunction(request)
            };

            // this doesn't work unfortunately
            //formatter.SerializerSettings = GlobalConfiguration.Configuration.Formatters.JsonFormatter.SerializerSettings;

            // You have to reapply any JSON.NET default serializer Customizations here    
            formatter.SerializerSettings.Converters.Add(new StringEnumConverter());
            formatter.SerializerSettings.Formatting = Formatting.Indented;
            formatter.SerializerSettings.ContractResolver = CamelCasePropertyNamesContractResolver;

            return formatter;
        }


        public override async Task WriteToStreamAsync(Type type, object value,
                                        Stream stream,
                                        HttpContent content,
                                        TransportContext transportContext)
        {
            if (String.IsNullOrEmpty(_jsonpCallbackFunction)) {
                await base.WriteToStreamAsync(type, value, stream, content, transportContext);
            }
            else {
                await WrapCallback(type, value, stream, content, transportContext);
            }
        }

        private async Task WrapCallback(Type type, object value,
                                        Stream stream,
                                        HttpContent content,
                                        TransportContext transportContext)
        {
            using (var writer = new StreamWriter(stream))
            {
                writer.Write(_jsonpCallbackFunction + "(");
                writer.Flush();

                await base.WriteToStreamAsync(type, value, stream, content, transportContext);

                writer.Write(");//2");
                writer.Flush();
            }
        }

        /// <summary>
        /// Retrieves the Jsonp Callback function
        /// from the query string
        /// </summary>
        /// <returns></returns>
        private string GetJsonCallbackFunction(HttpRequestMessage request)
        {
            if (request.Method != HttpMethod.Get)
                return null;

            var query = HttpUtility.ParseQueryString(request.RequestUri.Query);
            var queryVal = query[JsonpParameterName];

            if (String.IsNullOrEmpty(queryVal))
                return null;

            return queryVal;
        }
    }
}